@@ -5,7 +5,7 @@ use Mouse;
 use XML::LibXML;
 use XML::LibXSLT;
 
-our $VERSION = '1.9.991_01';
+our $VERSION = '2.0.0';
 
 # Lemonldap::NG::Portal::Main::Plugin provides addAuthRoute() and
 # addUnauthRoute() methods in addition of Lemonldap::NG::Common::Module.
@@ -17,7 +17,7 @@ extends 'Lemonldap::NG::Portal::Main::Plugin';
 has parser => (
     is      => 'rw',
     builder => sub {
-        return XML::LibXML->new();
+        return XML::LibXML->new( load_ext_dtd => 0, expand_entities => 0 );
     }
 );
 
@@ -32,7 +32,7 @@ has stylesheet => (
           ( $self->conf->{notificationXSLTfile}
               and -e $self->conf->{notificationXSLTfile} )
           ? $self->conf->{notificationXSLTfile}
-          : $self->conf->{templatesDir} . '/common/notification.xsl';
+          : $self->conf->{templateDir} . '/common/notification.xsl';
         unless ( -e $styleFile ) {
             $self->{logger}->error("$styleFile not found, aborting");
             die "$styleFile not found";
@@ -66,7 +66,9 @@ sub checkForNotifications {
         my $xml = $self->parser->parse_string($file);
         my $j   = 0;                                    #Notifications count
       LOOP: foreach my $notif (
-            eval { $xml->documentElement->getElementsByTagName('notification') }
+            eval {
+                $xml->documentElement->getElementsByTagName('notification');
+            }
           )
         {
 
@@ -93,9 +95,18 @@ sub checkForNotifications {
             if ($condition) {
 
                 $self->logger->debug("Get condition $condition");
+                $condition = $self->p->HANDLER->substitute($condition);
+                unless ( $condition = $self->p->HANDLER->buildSub($condition) )
+                {
+                    $self->logger->error( 'Notification condition error: '
+                          . $self->p->HANDLER->tsv->{jail}->error );
+                    $notif->unbindNode();
+                    next LOOP;
+                }
 
-                unless ( $self->p->HANDLER->safe->reval($condition) ) {
-                    $self->logger->debug("Notification condition not accepted");
+                unless ( $condition->( $req, $req->sessionInfo ) ) {
+                    $self->logger->debug(
+                        'Notification condition not authorized');
 
                     # Remove it from XML
                     $notif->unbindNode();
@@ -124,7 +135,6 @@ sub checkForNotifications {
     return 0 unless $i;
 
     # Returns HTML fragment
-    $req->id( $self->p->HANDLER->tsv->{cipher}->encrypt( $req->id ) );
     return $form;
 }
 
@@ -143,11 +153,11 @@ sub getNotifBack {
       or return $self->sendError( $req, 'Unable to decrypt', 500 );
 
     # Verify that session exists
-    $req->userData( $self->p->HANDLER->retrieveSession($id) )
+    $req->userData( $self->p->HANDLER->retrieveSession( $req, $id ) )
       or return $self->sendError( $req, 'Unknown session', 401 );
 
-    # Restore datas
-    $self->p->importHandlerDatas($req);
+    # Restore data
+    $self->p->importHandlerData($req);
     my $uid = $req->sessionInfo->{ $self->notifObject->notifField };
 
     my ( $notifs, $forUser ) = $self->notifObject->getNotifications($uid);
@@ -199,9 +209,8 @@ sub getNotifBack {
                     # Current pending notification has not been found in
                     # request
                     $result = $fileResult = 0;
-                    $self->logger->debulogger->debug(
-                        'Current pending notification has not been found',
-                        'debug' );
+                    $self->logger->debug(
+                        'Current pending notification has not been found');
                     next;
                 }
 
@@ -211,8 +220,7 @@ sub getNotifBack {
                 $self->p->updatePersistentSession( $req,
                     { "notification_$reference" => time() } );
                 $self->logger->debug(
-                    "Notification $reference registered in persistent session"
-                );
+                    "Notification $reference registered in persistent session");
             }
 
         # Notifications accepted for this file, delete it unless it's a wildcard
@@ -225,10 +233,10 @@ sub getNotifBack {
 
             # One pending notification has been found and not accepted,
             # restart process to display pending notifications
-            # TODO: is it a good idea to launch all 'afterDatas' subs ?
+            # TODO: is it a good idea to launch all 'endAuth' subs ?
             $self->logger->debug(
                 'Pending notification has been found and not accepted');
-            return $self->p->do( $req, $self->p->afterDatas );
+            return $self->p->do( $req, [ @{ $self->p->endAuth } ] );
         }
 
         # All pending notifications have been accepted, restore cookies and
